codeunit 70030 "Intercept Ship Lines SBIESS"
{

    local procedure CheckMultipleShipTos(var SalesHeader: Record "Sales Header"; var TempShipTo: Record "Ship-to Address" temporary)
    var
        SalesLine: Record "Sales Line";
    begin
        SalesLine.SetRange("Document Type", SalesHeader."Document Type");
        SalesLine.SetRange("Document No.", SalesHeader."No.");
        SalesLine.SetFilter("Qty. to Ship", '<>0');
        if SalesLine.FindSet(false) then
            repeat
                if not TempShipTo.Get(SalesHeader."Sell-to Customer No.", SalesLine."Ship-To Code") then begin
                    TempShipTo."Customer No." := SalesHeader."Sell-to Customer No.";
                    TempShipTo.Code := SalesLine."Ship-To Code";
                    TempShipTo.Insert(false);
                end;
            until SalesLine.Next() = 0;
    end;

    local procedure UpdateSalesShipmentHeaderWithShipTo(var SalesShipmentHeader: Record "Sales Shipment Header"; Code: Code[10])
    var
        ShipToAddr: Record "Ship-to Address";
    begin
        SalesShipmentHeader."Ship-to Name" := ShipToAddr.Name;
        SalesShipmentHeader."Ship-to Name 2" := ShipToAddr."Name 2";
        SalesShipmentHeader."Ship-to Address" := ShipToAddr.Address;
        SalesShipmentHeader."Ship-to Address 2" := ShipToAddr."Address 2";
        SalesShipmentHeader."Ship-to City" := ShipToAddr.City;
        SalesShipmentHeader."Ship-to Post Code" := ShipToAddr."Post Code";
        SalesShipmentHeader."Ship-to County" := ShipToAddr.County;
        SalesShipmentHeader.Validate("Ship-to Country/Region Code", ShipToAddr."Country/Region Code");
        SalesShipmentHeader."Ship-to Contact" := ShipToAddr.Contact;
        if ShipToAddr."Shipment Method Code" <> '' then
            SalesShipmentHeader.Validate("Shipment Method Code", ShipToAddr."Shipment Method Code");
        if ShipToAddr."Location Code" <> '' then
            SalesShipmentHeader.Validate("Location Code", ShipToAddr."Location Code");
        SalesShipmentHeader."Shipping Agent Code" := ShipToAddr."Shipping Agent Code";
        SalesShipmentHeader."Shipping Agent Service Code" := ShipToAddr."Shipping Agent Service Code";
        if ShipToAddr."Tax Area Code" <> '' then
            SalesShipmentHeader."Tax Area Code" := ShipToAddr."Tax Area Code";
        SalesShipmentHeader."Tax Liable" := ShipToAddr."Tax Liable";
    end;

    [EventSubscriber(ObjectType::Codeunit, Codeunit::"Sales-Post", 'OnAfterSalesShptHeaderInsert', '', false, false)]
    local procedure AfterMainShipmentCreated(var SalesShipmentHeader: Record "Sales Shipment Header"; SalesHeader: Record "Sales Header"; SuppressCommit: Boolean; WhseShip: Boolean; WhseReceive: Boolean; var TempWhseShptHeader: Record "Warehouse Shipment Header"; var TempWhseRcptHeader: Record "Warehouse Receipt Header")
    var
        SalesShipmentHeader2: Record "Sales Shipment Header";
        TempShipTo: Record "Ship-to Address" temporary;
        NoSeriesMgt: Codeunit NoSeriesManagement;
        i: Integer;
    begin
        // Built a count of *unique* ship-to codes involved in this shipment posting
        CheckMultipleShipTos(SalesHeader, TempShipTo);

        // We *only* need to do special processing if there are multiple ship-tos (blank included!)
        if TempShipTo.Count() > 1 then begin
            TempShipTo.FindFirst();  // No error handling necessary, count made sure there is a record.

            //For the first record, we tag the 'standard shipment' with the ship-to code in our buffer
            SalesShipmentHeader."Split Shipment Main Shipment" := SalesShipmentHeader."No.";
            SalesShipmentHeader."Split Shipment Ship-To" := TempShipTo.Code;
            if TempShipTo.Code <> '' then
                UpdateSalesShipmentHeaderWithShipTo(SalesShipmentHeader, TempShipTo.Code);
            SalesShipmentHeader.Modify(true);
            TempShipTo.Next();

            // For subsequent Ship-Tos, we create new shipment headers
            repeat
                SalesShipmentHeader2.TransferFields(SalesShipmentHeader);
                SalesShipmentHeader2."No." := NoSeriesMgt.GetNextNo(SalesHeader."Shipping No. Series", SalesHeader."Posting Date", true);
                SalesShipmentHeader2."Split Shipment Main Shipment" := SalesShipmentHeader."No.";
                SalesShipmentHeader2."Split Shipment Ship-To" := TempShipTo.Code;
                if TempShipTo.Code <> '' then
                    UpdateSalesShipmentHeaderWithShipTo(SalesShipmentHeader, TempShipTo.Code);
                SalesShipmentHeader2.Insert(true);
            until TempShipTo.Next() = 0;
        end;

    end;

    [EventSubscriber(ObjectType::Table, Database::"Sales Shipment Line", 'OnAfterInitFromSalesLine', '', false, false)]
    local procedure RedirectShipmentLine(SalesShptHeader: Record "Sales Shipment Header"; SalesLine: Record "Sales Line"; var SalesShptLine: Record "Sales Shipment Line")
    var
        SalesShipmentHeader2: Record "Sales Shipment Header";
    begin
        if (SalesLine."Ship-To Code" = '') then
            exit;

        SalesShipmentHeader2.SetRange("Split Shipment Main Shipment", SalesShptHeader."No.");  // Make sure it's related to this posting
        SalesShipmentHeader2.SetRange("Split Shipment Ship-To", SalesLine."Ship-To Code");
        if SalesShipmentHeader2.FindFirst() then begin
            // we pull a quick 'redirect' move and make the line attached to an entirely different shipment document, tada!
            SalesShptLine."Document No." := SalesShipmentHeader2."No.";
        end;
    end;

}
